@@ -80,9 +80,7 @@ extern char* qs_get_selected_value(
 extern char* qs_alloc_zero_byte_string(
   request_rec* r);
 
-extern char* qs_trim_string(
-  request_rec* r, 
-  char*        s);
+extern char *qs_trim_string(apr_pool_t *p, char *s);
 
 extern int   chxj_chxjif_is_mine(
   device_table* spec, 
