@@ -96,8 +96,10 @@ cookie_t*
 chxj_save_cookie(request_rec* r)
 {
   int                 ii;
-  apr_array_header_t* headers;
-  apr_table_entry_t*  hentryp;
+  apr_array_header_t  *headers;
+  apr_table_entry_t   *hentryp;
+  apr_array_header_t  *err_headers;
+  apr_table_entry_t   *err_hentryp;
   char                *old_cookie_id;
   char                *store_string;
   mod_chxj_config     *dconf;
@@ -119,7 +121,7 @@ chxj_save_cookie(request_rec* r)
   has_cookie = 0;
   has_refer = 0;
 
-  dconf = ap_get_module_config(r->per_dir_config, &chxj_module);
+  dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
   entryp = chxj_apply_convrule(r, dconf->convrules);
   if (! entryp) {
     DBG(r, "end chxj_save_cookie() no pattern");
@@ -130,11 +132,10 @@ chxj_save_cookie(request_rec* r)
     return NULL;
   }
 
-
-
   headers = (apr_array_header_t*)apr_table_elts(r->headers_out);
   hentryp = (apr_table_entry_t*)headers->elts;
-
+  err_headers = (apr_array_header_t*)apr_table_elts(r->err_headers_out);
+  err_hentryp = (apr_table_entry_t*)err_headers->elts;
 
   new_cookie_table = apr_table_make(r->pool, 0);
 
@@ -155,7 +156,31 @@ chxj_save_cookie(request_rec* r)
         *val++ = 0;
         apr_table_add(new_cookie_table, key, val);
         if (strcasecmp(REFERER_COOKIE_KEY, key) == 0) has_refer++;
-        
+
+      }
+
+      has_cookie = 1;
+      DBG(r, "=====================================");
+    }
+  }
+  for (ii=0; ii<err_headers->nelts; ii++) {
+    if (strcasecmp(err_hentryp[ii].key, "Set-Cookie") == 0) {
+      DBG(r, "=====================================");
+      DBG(r, "cookie=[%s:%s]", err_hentryp[ii].key, err_hentryp[ii].val);
+
+      char* key;
+      char* val;
+      char* buff;
+
+
+      buff = apr_pstrdup(r->pool, err_hentryp[ii].val);
+      val = strchr(buff, '=');
+      if (val) {
+        key = buff;
+        *val++ = 0;
+        apr_table_add(new_cookie_table, key, val);
+        if (strcasecmp(REFERER_COOKIE_KEY, key) == 0) has_refer++;
+
       }
 
       has_cookie = 1;
@@ -163,16 +188,14 @@ chxj_save_cookie(request_rec* r)
     }
   }
   apr_table_unset(r->headers_out, "Set-Cookie");
+  apr_table_unset(r->err_headers_out, "Set-Cookie");
+
 
   if (! has_refer) {
     apr_uri_parse(r->pool,r->uri, &parsed_uri);
     refer_string = apr_psprintf(r->pool, 
                                 "%s://%s%s", 
-#if AP_SERVER_MAJORVERSION_NUMBER == 2 && AP_SERVER_MINORVERSION_NUMBER == 2
-                                ap_run_http_scheme(r),
-#else
-                                ap_run_http_method(r),
-#endif
+                                chxj_run_http_scheme(r),
                                 r->hostname,
                                 apr_uri_unparse(r->pool,
                                                 &parsed_uri,
@@ -306,7 +329,7 @@ chxj_update_cookie(request_rec* r, cookie_t* old_cookie)
   cookie->cookie_id = NULL;
 
 
-  dconf = ap_get_module_config(r->per_dir_config, &chxj_module);
+  dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
   entryp = chxj_apply_convrule(r, dconf->convrules);
   if (! entryp) {
     DBG(r, "end chxj_update_cookie() no pattern");
@@ -423,7 +446,7 @@ chxj_load_cookie(request_rec* r, char* cookie_id)
   cookie->cookie_headers = NULL;
   cookie->cookie_id = apr_pstrdup(r->pool, cookie_id);
 
-  dconf = ap_get_module_config(r->per_dir_config, &chxj_module);
+  dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
   entryp = chxj_apply_convrule(r, dconf->convrules);
   if (! entryp) {
     DBG(r, "end chxj_load_cookie() no pattern");
@@ -557,7 +580,7 @@ check_valid_cookie_attribute(request_rec *r, const char *value)
   for (;;) {
     pair = apr_strtok(NULL, ";", &pstat);
     if (! pair) break;
-    pair = qs_trim_string(r, pair);
+    pair = qs_trim_string(r->pool, pair);
     if (STRNCASEEQ('d','D',"domain", pair, sizeof("domain")-1)) {
       domain_pair = apr_pstrdup(r->pool, pair);
     }
@@ -616,9 +639,9 @@ valid_domain(request_rec *r, const char *value)
     return CHXJ_TRUE;
 
   name = apr_strtok(p,"=", &pstat);
-  name = qs_trim_string(r, name);
+  name = qs_trim_string(r->pool, name);
   val = apr_strtok(NULL, "=", &pstat);
-  val = qs_trim_string(r, val);
+  val = qs_trim_string(r->pool, val);
   len = strlen(host);
   if (len) {
     if (chxj_strcasenrcmp(r->pool, host, val, strlen(val))) {
@@ -666,8 +689,8 @@ valid_path(request_rec *r, const char *value)
   DBG(r, "uri=[%s]", uri);
   name = apr_strtok(p, "=", &pstat);
   val = apr_strtok(NULL, "=", &pstat);
-  name = qs_trim_string(r, name);
-  val = qs_trim_string(r, val);
+  name = qs_trim_string(r->pool, name);
+  val = qs_trim_string(r->pool, val);
   DBG(r, "name=[%s] val=[%s]", name, val);
   
   DBG(r, "val:[%s] vs uri:[%s]", val, uri);
@@ -695,7 +718,6 @@ valid_expires(request_rec *r, const char *value)
   DBG(r, "name=[%s] val=[%s]", name, val);
   now = apr_time_now();
   expires = chxj_parse_cookie_expires(val);
-  DBG(r, "now=[%lld] expires=[%lld]", now, expires);
   if (expires < now) {
     DBG(r, "end valid_expire() value:[%s] (expired)", value);
     return CHXJ_FALSE;
@@ -710,11 +732,7 @@ valid_secure(request_rec *r, const char *value)
 {
   const char *scheme;
   DBG(r, "start valid_secure() value:[%s]", value);
-#if AP_SERVER_MAJORVERSION_NUMBER == 2 && AP_SERVER_MINORVERSION_NUMBER == 2
-  scheme = ap_run_http_scheme(r);
-#else
-  scheme = ap_run_http_method(r);
-#endif
+  scheme = chxj_apache_run_http_scheme(r);
   if (strcasecmp("https", scheme)) {
     DBG(r, "end valid_secure() value:[%s] (non secure)", value);
     return CHXJ_FALSE;
@@ -822,7 +840,7 @@ chxj_delete_cookie(request_rec *r, const char *cookie_id)
   mod_chxj_config *dconf;
 
   DBG(r, "start chxj_delete_cookie()");
-  dconf = ap_get_module_config(r->per_dir_config, &chxj_module);
+  dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
 
 #if defined(USE_MYSQL_COOKIE)
   if (IS_COOKIE_STORE_MYSQL(dconf->cookie_store_type)) {
@@ -876,7 +894,7 @@ chxj_save_cookie_expire(request_rec* r, cookie_t* cookie)
     return;
   }
 
-  dconf = ap_get_module_config(r->per_dir_config, &chxj_module);
+  dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
   if (!dconf) {
     DBG(r, "dconf is NULL");
     return;
@@ -925,7 +943,7 @@ chxj_delete_cookie_expire(request_rec* r, char* cookie_id)
 
   DBG(r, "start chxj_delete_cookie_expire()");
 
-  dconf = ap_get_module_config(r->per_dir_config, &chxj_module);
+  dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
 
 #if defined(USE_MYSQL_COOKIE)
   if (IS_COOKIE_STORE_MYSQL(dconf->cookie_store_type)) {
@@ -966,7 +984,7 @@ chxj_cookie_expire_gc(request_rec* r)
 
   DBG(r, "start chxj_cookie_expire_gc()");
 
-  dconf = ap_get_module_config(r->per_dir_config, &chxj_module);
+  dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
 #if defined(USE_MYSQL_COOKIE)
   if (IS_COOKIE_STORE_MYSQL(dconf->cookie_store_type)) {
     if (! chxj_cookie_expire_gc_mysql(r, dconf)) {
